# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright © 2005-2010 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later
from gettext import gettext as _

pname = 'Griffith'
pversion = '0.24'
pauthor = 'Vasco Nunes, Piotr Ożarowski <griffith@griffith.cc>, Dan Johansen <strit@strits.dk>'
pyear = '2005-2024'
pwebsite = 'https://gitlab.com/Strit/griffith/'
pdescription = _('Griffith is a film collection manager.')
